(*<optonet.pas>
 * Low level network implementation for OPTO-TECH multi-card
 * Network/RS232/SD-Card for MSX platform.
 * CopyLeft (c) since 2013 by PopolonY2k.
 *)

(**
  *
  * $Id: optonet.pas 98 2015-08-21 01:28:40Z popolony2k $
  * $Author: popolony2k $
  * $Date: 2015-08-21 01:28:40 +0000 (Fri, 21 Aug 2015) $
  * $Revision: 98 $
  * $HeadURL: file:///svn/p/oldskooltech/code/msx/trunk/msxdos/pascal/optonet.pas $
  *)

(*
 * This module depends on folowing include files (respect the order):
 * - sockdefs.pas;
 * - types.pas;
 * - helpstr.pas;
 *)

(*
 * Internal addresses and commands used by OptoNet cards.
 *)
Const      ctConfigPort           : Byte = $90;  { Card config port }
           ctDataPort             : Byte = $91;  { Card data port }
           { Ethernet commands }
           ctCMDRequestBufferSize : Byte = 0;    { Get the buffer size }
           ctCMDClearBuffers      : Byte = 20;   { Clear all buffers }
           ctCMDSetIPAddr         : Byte = 30;   { Set a new board IP address }
           ctCMDSetGatewayAddr    : Byte = 35;   { Set gateway IP address }
           ctCMDResetIPAddr       : Byte = 40;   { Reset to default IP address }
           ctCMDSetRemoteIPAddr   : Byte = 45;   { Set remote IP address }
           ctCMDSetPort           : Byte = 47;   { Set new port. Default 10001 }
           ctCMDSendUDPPacket     : Byte = 50;   { Send a UDP packet }
           ctCMDResolveDNS        : Byte = 60;   { Resolve DNS }
           { Miscelaneous }
           ctCMDResetBoard        : Byte = 43;   { Reset the board }
           { Serial communication }
           ctCMDUARTInit1200      : Byte = 1;    { Init UART 1200 baud rate }
           ctCMDUARTInit2400      : Byte = 2;    { Init UART 2400 baud rate }
           ctCMDUARTInit4800      : Byte = 4;    { Init UART 4800 baud rate }
           ctCMDUARTInit9600      : Byte = 9;    { Init UART 9600 baud rate }
           ctCMDUARTInit19200     : Byte = 19;   { Init UART 19200 baud rate }
           ctCMDUARTInit38400     : Byte = 38;   { Init UART 38400 baud rate }
           ctCMDUARTInit57600     : Byte = 57;   { Init UART 57600 baud rate }
           ctCMDUARTInit115200    : Byte = 115;  { Init UART 115200 baud rate }
           ctCMDUARTInit256000    : Byte = 250;  { Init UART 256000 baud rate }
           ctCMDSendSerialPacket  : Byte = 55;   { Send a packet over serial }
           { SD Card communication }
           ctCMDSDCardOn          : Byte = 10;   { Turn SDCard On and disable }
                                                 { serial and ethernet modes }
           (*
            * FIXME:
            * Wait used just when there bugs on Firmware.
            * These constants and the wait call will be removed at the end
            * of development.
            *)
           ctOutPortWait          : Byte = 3;    { Output port wait }
           ctCommandPortWait      : Byte = 8;    { Command port wait }


(* Low level board functions. Don't use this directly. *)

(**
  * Write one byte to the specified port.
  * @param nPort The port to send the data;
  * @param nData The Data to send over the port;
  *)
Procedure __OptoWritePort( nPort : Integer; nData : Byte );
Begin
  Port[nPort] := nData;
  Delay( ctOutPortWait );   { FIXME: }
End;

(**
  * Read a information from data the OPTONET data port;
  *)
Function __OptoReadPort : Byte;
Begin
  __OptoReadPort := Port[ctDataPort];
End;

(**
  * Reset the network card.
  *)
Procedure __OptoResetBoard;
Begin
  __OptoWritePort( ctConfigPort, ctCMDResetBoard );

  (*
   * FIXME:
   * The Wait() below exist because there problems on the current
   * firmware to process commands at full speed call.
   * This will be fixed until the end of OptoNet network development.
   *)
  Delay ( ctCommandPortWait );
End;

(**
  * Clear the network card buffers.
  *)
Procedure __OptoClearBuffers;
Begin
  __OptoWritePort( ctConfigPort, ctCMDClearBuffers );

  (*
   * FIXME:
   * The Wait() below exist because there problems on the current
   * firmware to process commands at full speed call.
   * This will be fixed until the end of OptoNet network development.
   *)
  Delay ( ctCommandPortWait );
End;

(**
  * Set the port to use in the next call to @link __OptoSetAddress();
  * @param nPort The port to set;
  *)
Function __OptoSetPort( nPort : Integer ) : TSocketResult;
Begin
  __OptoClearBuffers;
  __OptoWritePort( ctDataPort, Hi( nPort ) );   { Local port }
  __OptoWritePort( ctDataPort, Lo( nPort ) );
  __OptoWritePort( ctDataPort, Hi( nPort ) );   { Remote port }
  __OptoWritePort( ctDataPort, Lo( nPort ) );
  __OptoWritePort( ctConfigPort, ctCMDSetPort );

  (*
   * FIXME:
   * The Wait() below exist because there problems on the current
   * firmware to process commands at full speed call.
   * This will be fixed until the end of OptoNet network development.
   *)
  Delay ( ctCommandPortWait );

  __OptoSetPort := SocketSuccess;
End;

(**
  * Set a IP address into the board.
  * @param nCMD A valid board command to set new IP;
  * @param strAddress A valid internet address to set on the board;
  *)
Function __OptoSetAddress( nCMD : Byte;
                           strAddress : TIPAddress ) : TSocketResult;
Var
     aStrIPAddr  : TStringArray;
     aIntIPAddr  : Array[0..3] Of Integer;
     nCount      : Byte;
     nCode       : Integer;
     ResultCode  : TSocketResult;

Begin
  nCount := Split( strAddress, '.', aStrIPAddr );

  If( nCount = 4 )  Then
  Begin
    ResultCode := SocketSuccess;
    nCount := 0;

    While( nCount < 4 ) Do
    Begin
      Val( aStrIPAddr[nCount], aIntIPAddr[nCount], nCode );

      If( nCode <> 0 )  Then
      Begin
        nCount := 4;
        ResultCode := SocketInvalidIP;
      End
      Else
        nCount := nCount + 1;
    End;
  End
  Else
    ResultCode := SocketInvalidIP;

  { Send command to the board }
  If( ResultCode = SocketSuccess )  Then
  Begin
    __OptoClearBuffers;

    For nCount := 0 To 3 Do
      __OptoWritePort( ctDataPort, aIntIPAddr[nCount] );

    __OptoWritePort( ctConfigPort, nCMD );

    (*
     * FIXME:
     * The Wait() below exist because there problems on the current
     * firmware to process commands at full speed call.
     * This will be fixed until the end of OptoNet network development.
     *)
    Delay ( ctCommandPortWait );

    __OptoClearBuffers;
  End;

  __OptoSetAddress := ResultCode;
End;

(**
  * Send a UDP data to the board.
  * @param strData The data to be sent;
  *)
Function __OptoSendUDPPacket( Var packet : TSocketPacket ) : TSocketResult;
Var
         nPacketAddress : Integer;
         nPacketSize,
         nCount         : Byte;
         ResultCode     : TSocketResult;

Begin
  If( packet.nSize > 0 )  Then
  Begin
    nPacketAddress := Ord( packet.pData );
    nPacketSize    := packet.nSize - 1;

    For nCount := 0 To nPacketSize Do
      __OptoWritePort( ctDataPort, Mem[nPacketAddress + nCount] );

    __OptoWritePort( ctConfigPort, ctCMDSendUDPPacket );

    ResultCode := SocketSuccess;

    (*
     * FIXME:
     * The Wait() below exist because there problems on the current
     * firmware to process commands at full speed call.
     * This will be fixed until the end of OptoNet network development.
     *)
    Delay ( ctCommandPortWait );
  End
  Else
    ResultCode := SocketInvalidPacket;

  __OptoSendUDPPacket := ResultCode;
End;

(**
  * Receive a UDP data from the board.
  * @param packet The data to be received;
  *)
Function __OptoRecvUDPPacket( Var packet : TSocketPacket ) : TSocketResult;
Var
         nPacketAddress : Integer;
         nPacketSize,
         nCount         : Byte;
         ResultCode     : TSocketResult;

Begin
  nPacketAddress := Ord( packet.pData );
  (* Request the buffer size to board *)
  __OptoWritePort( ctConfigPort, ctCMDRequestBufferSize );

  nPacketSize := __OptoReadPort;

  If( nPacketSize > 0 )  Then
  Begin
    packet.nSize := nPacketSize;
    nPacketSize  := nPacketSize - 1;

    For nCount := 0 To nPacketSize Do
      Mem[nPacketAddress + nCount] := __OptoReadPort;
  End;

  (*
   * FIXME:
   * The Wait() below exist because there problems on the current
   * firmware to process commands at full speed call.
   * This will be fixed until the end of OptoNet network development.
   *)
  Delay ( ctCommandPortWait );

  ResultCode := SocketSuccess;

  __OptoRecvUDPPacket := ResultCode;
End;

(*
 * Driver functions to provide abstract socket compatibility.
 *)

(**
  * Function provided by OptoNet driver to provide socket connection.
  * @param nDriverParms The pointer to the @see TDriverParms struct
  * containing the driver input and output parameters;
  *)
Procedure __OptoNetDrvConnect( nDriverParms : Integer );
Var
       pParms      : PDriverParms;
       pSock       : PSocket;
       pResult     : PSocketResult;
Begin
  pParms := Ptr( nDriverParms );

  With pParms^ Do
  Begin
    pSock   := Ptr( nInParm );
    pResult := Ptr( nOutParm );

    With pSock^ Do
    Begin
      Connection.nSocketHandle := 0;
      pResult^ := __OptoSetPort( nPort );

      If( pResult^ = SocketSuccess )  Then
      Begin
        pResult^ := __OptoSetAddress( ctCMDSetRemoteIPAddr, strIPAddress );

        If( pResult^ = SocketSuccess )  Then
          Connection.nSocketHandle := 1;   { Simulating a valid handle }
      End;
    End;
  End;
End;

(**
  * Function provided by OptoNet driver to provide socket Disconnection.
  * @param nDriverParms The pointer to the @see TDriverParms struct
  * containing the driver input and output parameters;
  *)
Procedure __OptoNetDrvDisconnect( nDriverParms : Integer );
Var
       pParms      : PDriverParms;
       pSock       : PSocket;
       pResult     : PSocketResult;
Begin
  pParms := Ptr( nDriverParms );

  With pParms^ Do
  Begin
    pSock   := Ptr( nInParm );
    pResult := Ptr( nOutParm );

    With pSock^ Do
    Begin
      Connection.nSocketHandle := 0;
      pResult^ := SocketSuccess;
    End;
  End;
End;

(**
  * Function provided by OptoNet driver to provide send a information
  * through a socket.
  * @param nDriverParms The pointer to the @see TDriverParms struct
  * containing the driver input and output parameters;
  *)
Procedure __OptoNetDrvSendPacket( nDriverParms : Integer );
Var
       pParms      : PDriverParms;
       pPacket     : PSocketPacket;
       pResult     : PSocketResult;
Begin
  pParms := Ptr( nDriverParms );

  With pParms^ Do
  Begin
    pPacket  := Ptr( nInParm );
    pResult  := Ptr( nOutParm );

    If( pPacket^.pSock^.Connection.SocketType = SOCK_DGRAM )  Then
      pResult^ := __OptoSendUDPPacket( pPacket^ )
    Else
      pResult^ := SocketNotImplemented;
  End;
End;

(**
  * Function provided by OptoNet driver to provide receive a information
  * from the socket.
  * @param nDriverParms The pointer to the @see TDriverParms struct
  * containing the driver input and output parameters;
  *)
Procedure __OptoNetDrvRecvPacket( nDriverParms : Integer );
Var
       pParms      : PDriverParms;
       pPacket     : PSocketPacket;
       pResult     : PSocketResult;
Begin
  pParms := Ptr( nDriverParms );

  With pParms^ Do
  Begin
    pPacket := Ptr( nInParm );
    pResult := Ptr( nOutParm );

    If( pPacket^.pSock^.Connection.SocketType = SOCK_DGRAM )  Then
      pResult^ := __OptoRecvUDPPacket( pPacket^ )
    Else
      pResult^ := SocketNotImplemented;
  End;
End;

(**
  * Function provided by OptoNet driver to provide socket initialization.
  * @param nDriverParms The pointer to the @see TDriverParms struct
  * containing the driver input and output parameters;
  *)
Procedure OptoNetDrvSocketInit( nDriverParms : Integer );
Var
       pParms    : PDriverParms;
       pSock     : PSocket;
Begin
  pParms := Ptr( nDriverParms );

  With pParms^ Do
  Begin
    pSock := Ptr( nInParm );

    With pSock^ Do
    Begin
      DriverLayer.nConnectFn    := Addr( __OptoNetDrvConnect );
      DriverLayer.nDisconnectFn := Addr( __OptoNetDrvDisconnect );
      DriverLayer.nSendPacketFn := Addr( __OptoNetDrvSendPacket );
      DriverLayer.nRecvPacketFn := Addr( __OptoNetDrvRecvPacket );;
    End;
  End;
End;
