(*<vgmfile.pas>
 * Library for VGM music handling.
 * Check for newer VGM format definition at following URL address below
 * http://www.smspower.org/Music/VGMFileFormat?from=Development.VGMFormat
 * CopyLeft (c) since 1995 by PopolonY2k.
 *)

(**
  *
  * $Id: vgmfile.pas 98 2015-08-21 01:28:40Z popolony2k $
  * $Author: popolony2k $
  * $Date: 2015-08-21 01:28:40 +0000 (Fri, 21 Aug 2015) $
  * $Revision: 98 $
  * $HeadURL: file:///svn/p/oldskooltech/code/msx/trunk/msxdos/pascal/vgmfile.pas $
  *)

(*
 * This module depends on folowing include files (respect the order):
 * - popmem.pas;
 * - types.pas;
 * - msxdos.pas;
 * - msxdos2.pas;
 * - msxbios.pas;
 * - dos2file.pas;
 * - vgmtypes.pas;
 *)

(**
  * Header position index (Version 1.70). All fields are 32Bit, except when
  * the comments shows the field's real size.
  *)
Const
        ctFileIdentification    = $00; { File identification }
        ctEOFOffset             = $04; { End of File Offset  }
        ctVersionNumber         = $08; { VGM Version number  }
        ctTotalSamples          = $18; { Total samples }
        ctLoopOffset            = $1C; { Loop offset   }
        ctLoopSamples           = $20; { Loop samples  }
        ctRate                  = $24; { Rate          }
        ctVGMDataOffset         = $34; { VGM data offset }
        ctReserved1             = $7D; { Reserved - 8Bit }
        ctReserved2             = $97; { Reserved - 8Bit }
        ctReserved3             = $B8; { Reserverd }
        ctExtraHeaderOffset     = $BC; { Extra header offset }
        ctReserved4             = $C0; { Reserved - Free space }

        ctDiskSectorSize        = $80; { The disk sector size }


(**
  * Open a specified VGM file passed by parameter, returning the header
  * and VGM data content to play;
  * @param strFileName The file name that will be opened;
  * @param data The @see TVGMData structure containing the VGM
  * data to be played. Is something fails, the bValidContent flag will be
  * False;
  *)
Procedure OpenVGM( strFileName : TFileName; Var data : TVGMData );

(*
 * Internal local constants.
 *)
Const
         ctBufferSize = 127;  { The data buffer size }


(*
 * Local data.
 *)
Var
         hFile          : Byte;
         aDiskBuffer    : Array[0..ctBufferSize] Of Byte;
         pCurrentPosPtr : Pointer;
         nAllocSize     : Integer;
         bEndOfFile     : Boolean;


  (**
    * Move the header and data chunks to the internal buffer.
    *)
  Procedure __MoveChunks;
  Begin
    Move( aDiskBuffer, pCurrentPosPtr^, SizeOf( aDiskBuffer ) );
    pCurrentPosPtr := Ptr( Ord( pCurrentPosPtr ) + SizeOf( aDiskBuffer ) );
  End;

  (**
    * This procedure retrieves all mandatory fields accepted by all versions
    * of VGM format.
    *)
  Procedure __GetMandatoryFields;
  Begin
    (* EOF Offset *)
    GetData( Addr( aDiskBuffer ),
             Addr( data.header.nEOFOffset ),
             ctEOFOffset,
             SizeOf( data.header.nEOFOffset ) );

    (*
     * FileFormat Version.
     * For versions prior to 1.50, VGM data must start at offset 0x40 so in
     * this case the HeaderSize is 0x40 (64Bytes).
     *)
    GetData( Addr( aDiskBuffer ),
             Addr( data.header.nVersionNumber ),
             ctVersionNumber,
             SizeOf( data.header.nVersionNumber ) );

    (* GD3 Offset *)
    GetData( Addr( aDiskBuffer ),
             Addr( data.header.nGD3Offset ),
             ctGD3Offset,
             SizeOf( data.header.nGD3Offset ) );

    (* Total Samples *)
    GetData( Addr( aDiskBuffer ),
             Addr( data.header.nTotalSamples ),
             ctTotalSamples,
             SizeOf( data.header.nTotalSamples ) );

    (* Loop Offset *)
    GetData( Addr( aDiskBuffer ),
             Addr( data.header.nLoopOffset ),
             ctLoopOffset,
             SizeOf( data.header.nLoopOffset ) );

    (* Loop # samples *)
    GetData( Addr( aDiskBuffer ),
             Addr( data.header.nLoopSamples ),
             ctLoopSamples,
             SizeOf( data.header.nLoopSamples ) );

    (* Rate - Before 1.01 will be filled with zeroes *)
    GetData( Addr( aDiskBuffer ),
             Addr( data.header.nRate ),
             ctRate,
             SizeOf( data.header.nRate ) );

    (* VGM Data Offset - Before 1.01 will be filled with zeroes *)
    GetData( Addr( aDiskBuffer ),
             Addr( data.header.nVGMDataOffset ),
             ctVGMDataOffset,
             SizeOf( data.header.nVGMDataOffset ) );

    (* Calculates the Header size *)
    If( ( data.header.nVersionNumber[0] <= $50 ) And
        ( data.header.nVersionNumber[1] <= $1 ) )  Then
    Begin
      data.nHeaderSize := ctHeaderSizeVer100;
      data.nDataSize   := 0;
    End
    Else
    Begin
      (*
       * Calculates the HeaderSize used by protocol versions equals or higher
       * than 1.50;
       * Copy just the 16Bit part of VGMDataOffset because the library won't
       * handle large files containing 32Bit amount of data.
       *)
      Move( data.header.nVGMDataOffset,
            data.nHeaderSize,
            SizeOf( data.nHeaderSize ) );

      If( data.nHeaderSize > 0 )  Then
        data.nHeaderSize := data.nHeaderSize + ctVGMDataOffset;
    End;

    (* Calculates the DataSize *)
    Move( data.header.nEOFOffset,
          data.nDataSize,
          SizeOf( data.nDataSize ) );
    data.nDataSize := ( data.nDataSize - data.nHeaderSize - ctEOFOffset );
  End;

  (**
    * Check the file status looking for some errors or features that
    * the library yet can't handle.
    *)
  Procedure __CheckFileStatus;
  Begin
    (*
     * If the file is larger than 16Bit will be treated as error because
     * the library can't handle (yet) larger files with 32Bit capacity
     * of data storing.
     *)
    If( ( data.header.nEOFOffset[2] <> 0 ) Or
        ( data.header.nEOFOffset[3] <> 0 ) )  Then  { Just 16Bit }
      data.status := StateNotEnoughMemory
    Else
      If( data.nHeaderSize = 0 )  Then     { Invalid header data }
        data.status := StateInvalidHeaderData;
  End;


(*
 * Main entry point for OpenVGM procedure.
 *)
Begin
  data.status     := StateInvalidFile;  { The first processing state }
  data.pVGMBuffer := Nil;

  hFile := FileOpen( strFileName, 'r' );

  If( hFile <> ctInvalidFileHandle )  Then
  Begin
    Repeat
      (*
       * Process the VGM file.
       *)
      bEndOfFile := FileBlockRead( hFile,
                                   aDiskBuffer,
                                   SizeOf( aDiskBuffer ) ) = ctReadWriteError;
      If( Not bEndOfFile )  Then
      Begin
        case( data.status ) Of
          StateInvalidFile :
          Begin
            (*
             * Check the file identification in header.
             *)
            If( ( aDiskBuffer[0] = Byte( 'V' ) ) And
                ( aDiskBuffer[1] = Byte( 'g' ) ) And
                ( aDiskBuffer[2] = Byte( 'm' ) ) And
                ( aDiskBuffer[3] = Byte( ' ' ) ) )  Then
            Begin
              (*
               * Get all mandatory fields and ajust the Header and Data sizes.
               *
               *)
              __GetMandatoryFields;

              (*
               * Check the file status looking by errors and unimplemented
               * features.
               *)
              __CheckFileStatus;

              (*
               * Just allocates memory for data at beggining of data
               * processing.
               *)
              If( data.status = StateInvalidFile ) Then
              Begin
                (*
                 * Check there is memory available.
                 *)
                nAllocSize := ( Round( ( data.nDataSize + data.nHeaderSize ) /
                                        ctDiskSectorSize ) + 1 ) *
                                        ctDiskSectorSize;

                If( Abs( MemAvail ) >= Abs( nAllocSize ) )  Then
                Begin
                  data.status := StateSuccessfullyLoaded;
                  GetMem( data.pVGMBuffer, nAllocSize );
                End;

                (*
                 * Check if the allocation was sucessfully performed.
                 *)
                If( data.pVGMBuffer = Nil )  Then
                  data.status := StateNotEnoughMemory
                Else
                Begin
                  pCurrentPosPtr := Ptr( Ord( data.pVGMBuffer ) );

                  (* Get the VGM starting buffer array pointer *)
                  data.pVGMSongBuffer := Ptr( Ord( data.pVGMBuffer ) +
                                              data.nHeaderSize );

                  __MoveChunks; { Move chunk to memory }
                End;
              End;
            End
            Else
              data.status := StateInvalidFileFormat;
          End;

          StateSuccessfullyLoaded  :  __MoveChunks; { Move chunks to memory }
        End;
      End;
    Until( bEndOfFile Or ( data.status In [StateInvalidHeaderData,
                                           StateInvalidFile,
                                           StateInvalidFileFormat,
                                           StateNotEnoughMemory] ) );

    (* Close the previously opened file *)
    bEndOfFile := FileClose( hFile );
  End;
End;

(**
  * Closes the VGM opened by @see OpenVGM. Call this method is mandatory when
  * the user won't use the VGM data content anymore. This releases all allocated
  * memory for the VGM data.
  *)
Procedure CloseVGM( Var data : TVGMData );
Var
     nFreeSize  : Integer;

Begin
  With data Do
  Begin
    If( ( pVGMBuffer <> Nil ) And ( ( nDataSize + nHeaderSize ) <> 0 ) )  Then
    Begin
      nFreeSize := ( Round( ( nDataSize + nHeaderSize ) /
                            ctDiskSectorSize ) + 1 ) * ctDiskSectorSize;
      FreeMem( pVGMBuffer, nFreeSize );
      pVGMBuffer := Nil;
    End;
  End;
End;
